@@ -18,100 +18,126 @@ class StoreClient(object):
     def __init__(self, store_url, username=None, password=None, timeout=30):
         self.store_url = store_url
         self.store_query_url = store_url + 'search/'
+        self.user_plugins_url = None
         self.username = username
         self.password = password
         self.timeout = timeout
 
-    def get_plugin(self, plugin_name):
+    def get_plugins(self, search_params=None):
         """
-        Get a plugin's information (descriptors and parameters) given its ChRIS store
-        name.
-        """
-        search_params = {'name': plugin_name}
-        plugins = self.get_plugins(**search_params)
-        if plugins:
-            return plugins[0] # plugin names are unique
-
-    def get_plugins(self, **search_params):
-        """
-        Get the data (descriptors and parameters) of a list of plugins given query
-        search parameters. If no search parameters is given then return all plugins
-        in the store.
+        Get a paginated list of plugin data (descriptors) given query search parameters.
+        If no search parameters is given then get the default first page.
         """
         if search_params:
             collection = self._get(self.store_query_url, search_params)
         else:
             collection = self._get(self.store_url)
-        # follow the 'parameters' link relation in each collection document
-        return self._get_items_from_paginated_collections(collection, ['parameters'])
+        return self.get_data_from_collection(collection)
+
+    def get_plugin(self, name, version=None):
+        """
+        Get a plugin's data (descriptors) given its ChRIS store name and version.
+        If no version is passed then the latest version of the plugin according to
+        creation date is retrieved.
+        """
+        if version is not None:
+            search_params = {'name_exact': name, 'version': version}
+        else:
+            version = 'latest'
+            search_params = {'name_exact_latest': name}
+        result = self.get_plugins(search_params)
+        if result['data']:
+            return result['data'][0]  # combination of plugin name and version is unique
+        else:
+            raise StoreRequestException('Could not find plugin name %s with version %s.' %
+                                        (name, version))
+
+    def get_plugin_by_id(self, id):
+        """
+        Get a plugin's data (descriptors) given its ChRIS store id.
+        """
+        search_params = {'id': id}
+        result = self.get_plugins(search_params)
+        if result['data']:
+            return result['data'][0]  # plugin ids are unique
+        else:
+            raise StoreRequestException('Could not find plugin with id %s' % id)
+
+    def get_plugin_parameters(self, plugin_id, params=None):
+        """
+        Get a plugin's paginated parameters given its ChRIS store id.
+        """
+        collection = self._get(self.store_query_url, {'id': plugin_id})
+        if len(collection.items) == 0:
+            raise StoreRequestException('Could not find plugin with id: %s.' % plugin_id)
+        parameters_links = self._get_link_relation_urls(collection.items[0], 'parameters')
+        if parameters_links:
+            collection = self._get(parameters_links[0], params) # there can only be a single parameters link
+            return self.get_data_from_collection(collection)
+        return {'data': [], 'hasNextPage': False, 'hasPreviousPage': False}
 
     def get_authenticated_user_plugins(self):
         """
-        Get the data (descriptors and parameters) of the plugins owned by the
-        currently authenticated user.
+        Get the data (descriptors) of the plugins owned by the currently authenticated
+        user.
         """
-        collection = self._get(self.store_url)
-        return self._get_items_from_paginated_collections(collection, ['parameters'])
+        url = self.user_plugins_url
+        if url is None:
+            collection = self._get(self.store_url)
+            url = self._get_link_relation_urls(collection, "user_plugins")[0]
+            self.user_plugins_url = url
+        collection = self._get(url)
+        return self.get_data_from_collection(collection)
+
+    def get_data_from_collection(self, collection):
+        """
+        Get the result data dictionary from a collection object.
+        """
+        result = {'data': [], 'hasNextPage': False, 'hasPreviousPage': False}
+        for item in collection.items:
+            item_dict = self._get_item_descriptors(item)
+            result['data'].append(item_dict)
+        if self._get_link_relation_urls(collection, 'next'):
+            result['hasNextPage'] = True
+        if self._get_link_relation_urls(collection, 'previous'):
+            result['hasPreviousPage'] = True
+        return result
 
     def add_plugin(self, name, dock_image, descriptor_file, public_repo):
         """
         Add a new plugin to the ChRIS store.
         """
+        url = self.user_plugins_url
+        if url is None:
+            collection = self._get(self.store_url)
+            url = self._get_link_relation_urls(collection, "user_plugins")[0]
+            self.user_plugins_url = url
         data = {'name': name, 'dock_image': dock_image, 'public_repo': public_repo}
-        url = self.store_url
-        self._post(url, data, descriptor_file)
+        collection = self._post(url, data, descriptor_file)
+        result = self.get_data_from_collection(collection)
+        return result['data'][0]
 
-    def modify_plugin(self, name, dock_image, descriptor_file, public_repo, newname = ''):
+    def modify_plugin(self, id, dock_image, public_repo):
         """
         Modify an existing plugin in the ChRIS store.
         """
-        search_params = {'name': name}
+        search_params = {'id': id}
         collection = self._get(self.store_query_url, search_params)
         url = collection.items[0].href
-        if newname:
-            name = newname
-        data = {'name': name, 'dock_image': dock_image, 'public_repo': public_repo}
-        self._put(url, data, descriptor_file)
+        data = {'dock_image': dock_image, 'public_repo': public_repo}
+        collection = self._put(url, data)
+        result = self.get_data_from_collection(collection)
+        return result['data'][0]
 
-    def remove_plugin(self, name):
+    def remove_plugin(self, id):
         """
         Remove an existing plugin from the ChRIS store.
         """
-        search_params = {'name': name}
+        search_params = {'id': id}
         collection = self._get(self.store_query_url, search_params)
         url = collection.items[0].href
         self._delete(url)
 
-    def _get_items_from_paginated_collections(self, collection, follow_link_relations=[]):
-        """
-        Internal method to recursively get the data (descriptors and related item's
-        descriptors) of all the items in a linked list of paginated collections.
-        """
-        if 'next' in follow_link_relations:
-            follow_link_relations.remove('next')
-        item_list = []
-        # get the collection documents from all pages
-        collections = self._get_paginated_collections(collection)
-        for collection in collections:
-            item_dict_list = []
-            items = collection.items
-            # for each item get its data and the data of all related items in a
-            # depth-first search
-            for item in items:
-                item_dict = self._get_item_descriptors(item)
-                for link_relation in follow_link_relations:
-                    related_urls = self._get_link_relation_urls(item, link_relation)
-                    if related_urls and (link_relation not in item_dict):
-                        item_dict[link_relation] = []
-                    for url in related_urls:
-                        coll = self._get(url)
-                        item_dict[link_relation].extend(
-                            self._get_items_from_paginated_collections(
-                                coll, follow_link_relations))
-                item_dict_list.append(item_dict)
-            item_list.extend(item_dict_list)
-        return item_list
-
     def _get_item_descriptors(self, item):
         """
         Internal method to get an item's data (descriptors) in a dictionary.
@@ -122,21 +148,6 @@ class StoreClient(object):
             item_dict[descriptor.name] = descriptor.value
         return item_dict
 
-    def _get_paginated_collections(self, collection):
-        """
-        Internal method to get a list of paginated collection objects.
-        """
-        collections = []
-        while True:
-            collections.append(collection)
-            next_page_urls = self._get_link_relation_urls(collection, 'next')
-            if next_page_urls:
-                # there is only a single next page
-                collection = self._get(next_page_urls[0])
-            else:
-                break
-        return collections
-
     def _get(self, url, params=None):
         """
         Internal method to make a GET request to the ChRIS store.
@@ -153,13 +164,13 @@ class StoreClient(object):
             raise StoreRequestException(str(e))
         return self._get_collection_from_response(r)
 
-    def _post(self, url, data, descriptor_file):
+    def _post(self, url, data, descriptor_file=None):
         """
         Internal method to make a POST request to the ChRIS store.
         """
         return self._post_put(requests.post, url, data, descriptor_file)
 
-    def _put(self, url, data, descriptor_file):
+    def _put(self, url, data, descriptor_file=None):
         """
         Internal method to make a PUT request to the ChRIS store.
         """
@@ -179,11 +190,13 @@ class StoreClient(object):
         except (requests.exceptions.Timeout, requests.exceptions.RequestException) as e:
             raise StoreRequestException(str(e))
 
-    def _post_put(self, request_method, url, data, descriptor_file):
+    def _post_put(self, request_method, url, data, descriptor_file=None):
         """
         Internal method to make either a POST or PUT request to the ChRIS store.
         """
-        files = {'descriptor_file': descriptor_file}
+        files = None
+        if descriptor_file is not None:
+            files = {'descriptor_file': descriptor_file}
         try:
             if self.username or self.password:
                 r = request_method(url, files=files, data=data,
